@@ -1,9 +1,11 @@
 import json
 import logging
+from types import SimpleNamespace
 from typing import Collection, Optional, Set, Tuple
 
 import colorama
 import jsonpatch
+import pydantic
 import square.k8s as k8s
 import square.manio as manio
 import square.schemas
@@ -11,14 +13,58 @@ import yaml
 from colorlog import ColoredFormatter
 from square.dtypes import (
     Config, DeltaCreate, DeltaDelete, DeltaPatch, DeploymentPlan,
-    DeploymentPlanMeta, JsonPatch, K8sConfig, MetaManifest, Selectors,
-    ServerManifests,
+    DeploymentPlanMeta, Filepath, JsonPatch, K8sConfig, MetaManifest,
+    Selectors, ServerManifests,
 )
 
 # Convenience: global logger instance to avoid repetitive code.
 logit = logging.getLogger("square")
 
 
+def load_config(fname: Filepath) -> Tuple[Config, bool]:
+    """Parse the Square configuration file `fname` and return it as a `Config`."""
+    err_resp = Config(Filepath(""), kubeconfig=Filepath(""), kubecontext=None), True
+    fname = Filepath(fname)
+
+    # Load the configuration file.
+    try:
+        raw = yaml.safe_load(Filepath(fname).read_text())
+    except FileNotFoundError as e:
+        logit.error(f"Cannot load config file <{fname}>: {e.args[1]}")
+        return err_resp
+    except yaml.YAMLError as exc:
+        msg = f"Could not parse YAML file {fname}"
+
+        # Special case: parser supplied location information.
+        mark = getattr(exc, "problem_mark", SimpleNamespace(line=-1, column=-1))
+        line, col = (mark.line + 1, mark.column + 1)
+        msg = f"YAML format error in {fname}: Line {line} Column {col}"
+        logit.error(msg)
+        return err_resp
+
+    if not isinstance(raw, dict):
+        logit.error(f"Config file <{fname}> has invalid structure")
+        return err_resp
+
+    # Parse the configuration into `ConfigFile` structure.
+    try:
+        cfg = Config(**raw)
+    except (pydantic.ValidationError, TypeError) as e:
+        logit.error(f"Schema is invalid: {e}")
+        return err_resp
+
+    # Remove the "_common_" filter and merge it into all the other filters.
+    common = cfg.filters.pop("_common_", [])
+    cfg.filters = {k: square.schemas.merge(common, v) for k, v in cfg.filters.items()}
+
+    # Make the necessary adjustments where the values in the file do not
+    # translate verbatim.
+    cfg.folder = fname.parent.absolute() / cfg.folder
+    cfg.selectors.kinds = set(cfg.selectors.kinds)
+    cfg.selectors.labels = {(k, v) for k, v in cfg.selectors.labels}
+    return cfg, False
+
+
 def make_patch(
         config: Config,
         k8sconfig: K8sConfig,
@@ -40,8 +86,8 @@ def make_patch(
     """
     # Reduce local and server manifests to salient fields (ie apiVersion, kind,
     # metadata and spec). Abort on error.
-    (loc, _), err1 = manio.strip(k8sconfig, local, square.schemas.EXCLUSION_SCHEMA)
-    (srv, _), err2 = manio.strip(k8sconfig, server, square.schemas.EXCLUSION_SCHEMA)
+    loc, _, err1 = manio.strip(k8sconfig, local, config.filters)
+    srv, _, err2 = manio.strip(k8sconfig, server, config.filters)
     if err1 or err2 or loc is None or srv is None:
         return (JsonPatch("", []), True)
 
@@ -275,9 +321,9 @@ def show_plan(plan: Optional[DeploymentPlan]) -> bool:
 
     # Terminal colours for convenience.
     cAdd = colorama.Fore.GREEN
+    cMod = colorama.Fore.YELLOW + colorama.Style.BRIGHT
     cDel = colorama.Fore.RED
-    cMod = colorama.Fore.YELLOW
-    cReset = colorama.Fore.RESET
+    cReset = colorama.Fore.RESET + colorama.Style.RESET_ALL
 
     n_add, n_mod, n_del = 0, 0, 0
 
@@ -328,11 +374,16 @@ def show_plan(plan: Optional[DeploymentPlan]) -> bool:
         print(cDel + f"Delete {name}" + cReset)
         n_del += 1
 
+    # Only use color if a category (ie to ADD, MODIFY or DELETE) is nonzero.
+    cAdd = cAdd if len(plan.create) else colorama.Style.BRIGHT + colorama.Fore.WHITE
+    cMod = cMod if len(plan.patch) else colorama.Style.BRIGHT + colorama.Fore.WHITE
+    cDel = cDel if len(plan.delete) else colorama.Style.BRIGHT + colorama.Fore.WHITE
+
     print("-" * 80)
     print("Plan: " +                         # noqa
-          cAdd + f"{n_add:,} to add, " +     # noqa
-          cMod + f"{n_mod:,} to change, " +  # noqa
-          cDel + f"{n_del:,} to destroy." +  # noqa
+          cReset + cAdd + f"{n_add:,} to add, " +     # noqa
+          cReset + cMod + f"{n_mod:,} to change, " +  # noqa
+          cReset + cDel + f"{n_del:,} to destroy." +  # noqa
           cReset + "\n")
     return False
 
@@ -430,7 +481,7 @@ def apply_plan(cfg: Config, plan: DeploymentPlan) -> bool:
     """
     try:
         # Create properly configured Requests session to talk to K8s API.
-        k8sconfig, err = k8s.cluster_config(cfg.kubeconfig, cfg.kube_ctx)
+        k8sconfig, err = k8s.cluster_config(cfg.kubeconfig, cfg.kubecontext)
         assert not err
 
         # Create the missing resources.
@@ -470,7 +521,7 @@ def make_plan(cfg: Config) -> Tuple[DeploymentPlan, bool]:
     """
     try:
         # Create properly configured Requests session to talk to K8s API.
-        k8sconfig, err = k8s.cluster_config(cfg.kubeconfig, cfg.kube_ctx)
+        k8sconfig, err = k8s.cluster_config(cfg.kubeconfig, cfg.kubecontext)
         assert not err
 
         # Load manifests from local files.
@@ -499,7 +550,7 @@ def get_resources(cfg: Config) -> bool:
     """Download all K8s manifests and merge them into local files."""
     try:
         # Create properly configured Requests session to talk to K8s API.
-        k8sconfig, err = k8s.cluster_config(cfg.kubeconfig, cfg.kube_ctx)
+        k8sconfig, err = k8s.cluster_config(cfg.kubeconfig, cfg.kubecontext)
         assert not err
 
         # Use a wildcard Selector to ensure `manio.load` will read _all_ local
@@ -508,9 +559,7 @@ def get_resources(cfg: Config) -> bool:
         # write the new ones. This logic will ensure we never have stale manifests
         # (see `manio.save_files` for details and how `manio.save`, which we call
         # at the end of this function, uses it).
-        load_selectors = Selectors(kinds=k8sconfig.kinds,
-                                   labels=None,
-                                   namespaces=None)
+        load_selectors = Selectors(kinds=k8sconfig.kinds, labels=set(), namespaces=[])
 
         # Load manifests from local files.
         _, local, err = manio.load(cfg.folder, load_selectors)
@@ -523,7 +572,7 @@ def get_resources(cfg: Config) -> bool:
         # Sync the server manifests into the local manifests. All this happens in
         # memory and no files will be modified here - see `manio.save` in the next step.
         synced_manifests, err = manio.sync(local, server, cfg.selectors, cfg.groupby)
-        assert not err and synced_manifests
+        assert not err
 
         # Write the new manifest files.
         err = manio.save(cfg.folder, synced_manifests, cfg.priorities)
